"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import lxml

class main:

	def __init__(self, q, limit=15):
		""" pubmed.ncbi.nlm.nih.gov search engine

				q         : query for search
				limit     : maximum result count
		"""
		self.framework = main.framework
		self.q = q
		self.max = limit
		self._rawhtml = ''
		self._articles = []
		self._links = []
		self._results = []
		self.pubmed = 'https://pubmed.ncbi.nlm.nih.gov'

	def run_crawl(self):
		self.framework.verbose('Searching the pubmed domain...')

		payload = {'term': self.q, 'size': 200}
		try:
			req = self.framework.request(
					url=self.pubmed,
					params=payload)
		except Exception as e:
			self.framework.error(f"ConnectionError {e}.", 'util/pubmed', 'run_crawl')
			self.framework.error('Pubmed is missed!', 'util/pubmed', 'run_crawl')
			return
		self._rawhtml += req.text
		doc = lxml.html.document_fromstring(req.text)
		self._articles = doc.findall('.//article')

	@property
	def raw(self):
		return self._rawhtml

	@property
	def articles(self):
		return self._articles

	@property
	def results(self):
		findlink = lambda x: self.pubmed + x.find_class('docsum-title')[0].attrib['href']
		findauthors = lambda x: x.find('.//span[@class="docsum-authors full-authors"]').text_content()
		findtitle = lambda x: x.find_class('docsum-title')[0].text_content().strip()
		findsummary = lambda x: x.find_class('full-view-snippet')[0].text_content().strip()

		for count,article in enumerate(self._articles):
			if count==self.max:
				break
			d = findsummary(article)
			self._results.append({
				't': findtitle(article),
				'a': findlink(article),
				'c': findauthors(article),
				'd': d if len(d)>0 else 'No abstract available'
				})

		return self._results



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from lxml import etree

class main:

	def __init__(self, q, limit=15):
		""" arxiv.org search engine

				q         : query for search
				limit     : maximum result count
		"""
		self.framework = main.framework
		self.q = q
		self.max = limit
		self._rawxml = ''
		self._tree = ''
		self._articles = []
		self._links = []
		self._results = []
		self.url = 'https://export.arxiv.org/api/query'
		self.NSMAP = {'w3':'http://www.w3.org/2005/Atom'}

	def run_crawl(self):
		payload = {'search_query': f"all:{self.q}", 'start': '0', 'max_results': self.max}
		self.framework.verbose('[ARXIV] Searching the arxiv.org domain...')
		try:
			req = self.framework.request(
					url=self.url,
					params=payload)
		except:
			self.framework.error('ConnectionError', 'arxiv', 'run_crawl')
			self.framework.error('ArXiv is missed!', 'arxiv', 'run_crawl')
			return
		self._rawxml = req.text
		self._tree = etree.fromstring(req.text.encode('utf-8'))
		self._articles = self.find(self._tree, './/w3:entry')


	def find(self, x, tofind):
		return x.findall(tofind, namespaces=self.NSMAP)

	@property
	def raw(self):
		return self._rawxml

	@property
	def articles(self):
		return self._articles

	@property
	def links(self):
		self._links = list(map(lambda x: x.text, 
			self.find(self._tree, './/w3:id')))
		return self._links

	@property
	def results(self):
		findlink = lambda x: self.find(x, './/w3:id')[0].text
		findauthors = lambda x: ', '.join(list(map(lambda x: x.text, 
				self.find(x, './/w3:author/w3:name'))))
		findtitle = lambda x: self.find(x, './/w3:title')[0].text
		findsummary = lambda x: self.find(x, './/w3:summary')[0].text.strip()

		for article in self._articles:
			self._results.append({
				't': findtitle(article),
				'a': findlink(article),
				'c': findauthors(article),
				'd': findsummary(article)
				})

		return self._results



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
import hashlib
import html
from bs4 import BeautifulSoup as bs

class main:

		def __init__(self, num):
			""" numverify search engine
					num     : number to search
			"""
			self.framework = main.framework
			self.num = num
			self._json = ''
			self._suffix = ''
			self.numverify = 'https://numverify.com'

		def forge_secret(self):
			# Numverify uses AJAX with an insecure hash to verify its requests
			# A secret value from a tag is appended to the phone number and md5'd 
			return hashlib.md5((self.num+self._suffix).encode()).hexdigest()

		def run_crawl(self):
			self.framework.verbose('Searching...')
			self.num = re.sub(r'\+|\s', '', self.num);
			try:
				req = self.framework.request(url=self.numverify)
				soup = bs(req.text,'html.parser')

				self._suffix = soup.find('input', {'name': 'scl_request_secret'})['value']
				secret = self.forge_secret()
				url = ''.join([f'{self.numverify}/php_helper_scripts/phone_api.php',
					f'?secret_key={secret}&number={self.num}'])

				req = self.framework.request(url=url)
				self._json = req.json()
			except Exception as e:
				self.framework.error(f"ConnectionError {e}.", 'util/numverify', 'run_crawl')
				self.framework.error('Numverify is missed!', 'util/numverify', 'run_crawl')
				return

		@property
		def json(self):
			return self._json



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

class main:

	def __init__(self, q, limit=2):
		""" millionshort.com search engine

			q 		  : query for search
			limit	  : Number of pages
		"""
		self.framework = main.framework
		self.q = q
		self.limit = limit
		self._pages = ''
		self._json = []
		self._links = []
		self._links_with_title = {}
		self.millionshort = 'millionshort.com'

	def run_crawl(self):
		payloads = {'keywords': self.q, 'remove': '0', 'offset': '0'}
		baseurl = f"https://{self.millionshort}/api/search"
		set_page = lambda x: x*10
		urls = [f'https://{self.millionshort}/api/search?keywords={self.q}&remove=0&offset={set_page(i)}'
					for i in range(self.limit)]
		for url in range(len(urls)):
			self.framework.verbose(f"[MILLIONSHORT] Searching in {url} page...")
			try:
				req = self.framework.request(url=urls[url])
			except:
				self.framework.error('ConnectionError')
			else:
				if 'captcha' in req.json():
					self.framework.error('CaptchaError', 'util/millionshort', 'run_crawl')
					return
				self._pages += req.text
				self._json.append(req.json())

	@property
	def pages(self):
		return self._pages

	@property
	def json(self):
		return self._json

	@property
	def links(self):
		for page in self.json:
			results = page.get('content', {}).get('webPages', {})
			self._links.extend([x.get('displayUrl') for x in results])
		return self._links

	@property
	def links_with_title(self):
		json = self.json
		if not json:
			return {}
		for page in self.json:
			results = page.get('content', {}).get('webPages', {})
			self._links_with_title.update({x.get('name'): x.get('displayUrl') for x in results})
		return self._links_with_title

	@property
	def dns(self):
		return self.framework.page_parse(self._pages).get_dns(self.q, self.links)

	@property
	def emails(self):
		return self.framework.page_parse(self._pages).get_emails(self.q)

	@property
	def docs(self):
		return self.framework.page_parse(self._pages).get_docs(self.q, self.links)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


meta = {
	'name': 'Qwant Search',
	'author': 'Saeed',
	'version': '0.2',
	'description': 'Search your query in the qwant.com and show the results(without limit).',
	'sources': ('qwant',),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
		('method', 'webpages', False, 'Qwant methods("webpages",\
		 "images", "news", "videos"). default=None', '-m', 'store', str),
		('limit', 2, False, 'Search limit(number of pages, default=2)', '-l', 'store', int),
	),
	'examples': ('qwant -q <QUERY>', 'qwant -q <QUERY> -m images -l 3')
}

def module_api(self):
	query = self.options['query']
	run = self.qwant(query, self.options['limit'])
	method = self.options['method'].lower()
	if method not in ('webpages', 'images', 'news', 'videos'):
		method = 'webpages'

	run.run_crawl(method)
	output = {'links': run.links_with_title}
	self.save_gather(output, 'search/qwant', query, output=self.options['output'])
	return output

def module_run(self):
	output = module_api(self)
	for item in output['links']:
		title, link = item[0], item[1]
		self.output(title)
		self.output(f"\t{link}", 'G')



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


import re

meta = {
	'name': 'Pastebin Search',
	'author': 'Divya Goswami',
	'version': '1.0',
	'description': 'Search your query in the pastebin.com and show the results as paste links.',
	'sources': ('pastebin',),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
		('limit', 1, False, 'Search limit(number of pages, default=1)', '-l', 'store', int),
		('count', 50, False, 'Number of results per page(min=10, max=100, default=50)', '-c', 'store', int),
		('thread', 3, False, 'The number of engine that run per round(default=2)', '-t', 'store', int),
		('engine', 'pastebin', False, 'Engine names for search', '-e', 'store', str),
	),
	'examples': ('pastebin -q passwords -l 15 --output',)
}

LINKS = []
PAGES = ''

def search(self, name, q, q_formats, limit, count):
	global PAGES,LINKS
	engine = getattr(self, name)
	q = q_formats[f"{name}_q"] if f"{name}_q" in q_formats else q_formats['default_q']
	varnames = engine.__init__.__code__.co_varnames
	if 'limit' in varnames and 'count' in varnames:
		attr = engine(q, limit, count)
	elif 'limit' in varnames:
		attr = engine(q, limit)
	else:
		attr = engine(q)

	attr.run_crawl()
	LINKS += attr.links
	PAGES += attr.pages

def module_api(self):
	query = self.options['query']
	limit = self.options['limit']
	count = self.options['count']
	engine = self.options['engine'].split(',')
	output = {'pastes': []}
	q_formats = {
		'default_q': f"{query}"
	}
	self.thread(search, self.options['thread'], engine, query, q_formats, limit, count, meta['sources'])
	links = list(self.reglib().filter(r"https?://pastebin\.com/[\w\d]{2,}", list(set(LINKS))))
	self.verbose('Rearranging paste links [give it a few seconds]...')
	for link in links:
		heading = re.search(r"pastebin\.com/([\w\d]+)", link)
		if heading:
			head_raw = f"https://pastebin.com/raw/{heading.group(1)}"
			try:
				head_req = self.request(url=head_raw).text.splitlines()[0].lstrip()
			except Exception as e:
				self.verbose('Pastebin is missed!')
			else:
				head_title = f"{query} pastes {head_req[:30]}...".ljust(10, ' ')
				title = head_title.title()
		output['pastes'].append([link, title])
	self.save_gather(output, 'search/pastebin', query, output=self.options.get('output'))
	return output

def module_run(self):
	output = module_api(self)
	for i in range(len(output['pastes'])):
		self.output(f"{output['pastes'][i][1]}", 'G')
		self.output(f"\t{output['pastes'][i][0]}", 'N')
	output.pop('pastes')
	self.alert_results(output)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

class main:

	def __init__(self, q, limit=2):
		""" qwant.com search engine

			q 		  : Query for search
			limit	  : Number of pages
		"""
		self.framework = main.framework
		self.q = q
		self.limit = limit
		self._pages = ''
		self._json = []
		self._links = []
		self._links_with_title = []
		self.qwant = 'qwant.com'

	def run_crawl(self, method='webpages'):
		policies = {'webpages': 'web',
					'images': 'images',
					'news': 'news',
					'videos': 'videos'}

		method = method.lower()
		if method not in policies:
			search_type = policies.get('webpages')
		else:
			search_type = policies[method]
		set_page = lambda x: x*10
		urls = [f'https://api.{self.qwant}/api/search/{search_type}?q={self.q}&t=web&device=desktop&safesearch=1&uiv=4&count=10&offset={set_page(i)}' 
					for i in range(self.limit)]
		headers = {'Host': 'api.qwant.com', 
				   'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64; rv:68.0) Gecko/20100101 Firefox/68.0',
				   'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8', 
				   'Accept-Language': 'en-US,en;q=0.5en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate, br', 
				   'Connection': 'keep-alive', 'Cookie': 'JSESSIONID=6120E7C52197190DE5126DCBF47D38B0', 
				   'Upgrade-Insecure-Requests': '1', 'Cache-Control': 'max-age=0'}
		for url in range(len(urls)):
			self.framework.verbose(f"[QWANT] Searching in {url} page...")
			try:
				req = self.framework.request(url=urls[url], headers=headers)
			except Exception as e:
				self.framework.error(f"ConnectionError {e}.", 'util/qwant', 'name_crawl')
				self.framework.error('Qwant is missed!', 'util/qwant', 'name_crawl')
			else:
				if req.status_code == 429 and "I can't let you do that..." in req.text and '<div class="error-code">' in req.text:
					self.framework.error('429 Too Many Requests')
					return
				self._pages += req.text
				try:
					self._json.append(req.json())
				except Exception as e:
					self.framework.error('429 Too Many Requests')
					return
				else:
					if req.json() == {"status": "error", "error": 22}:
						self.framework.error('429 Too Many Requests')

	@property
	def pages(self):
		return self._pages

	@property
	def json(self):
		return self._json

	@property
	def links(self):
		for page in self.json:
			results = page.get('data', {}).get('result', {}).get('items', {})
			self._links.extend([x.get('url') for x in results])
		return self._links

	@property
	def links_with_title(self):
		for page in self.json:
			items = page.get('data', {}).get('result', {}).get('items', {})
			if not items:
				return []
			for item in items:
				self._links_with_title.append([item['title'].replace('<b>','').replace('</b>', ''), item['url']])
		return self._links_with_title

	@property
	def dns(self):
		return self.framework.page_parse(self._pages).get_dns(self.q, self.links)

	@property
	def emails(self):
		return self.framework.page_parse(self._pages).get_emails(self.q)

	@property
	def docs(self):
		return self.framework.page_parse(self._pages).get_docs(self.q, self.links)



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

meta = {
	'name': 'DuckDuckGo',
	'author': 'Tarunesh Kumar',
	'version': '0.1',
	'description': 'Search your query in the duckduckgo.com and show the results.',
	'sources': ('duckduckgo',),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
		('limit', 1, False, 'Search limit(number of pages, default=1)', '-l', 'store', int),
		('count', 50, False, 'Number of results per page(min=10, max=100, default=50)', '-c', 'store', int),
	),
    'examples': ('duckduckgo -q <QUERY> -l 15 --output',)
}

def module_api(self):
	query = self.options['query']
	limit = self.options['limit']
	count = self.options['count']
	run = self.duckduckgo(query, limit, count)
	run.run_crawl()
	output = run.links_with_title
	links = run.links_with_title
	if abs(len(output) - len(links)) > 4:
		output = links
	self.save_gather({'links': output}, 'search/duckduckgo', query, output=self.options['output'])
	return output

def module_run(self):
	output = module_api(self)
	for item in output:
		title, link = item[1], item[0]
		self.output(title)
		self.output(f"\t{link}", 'G')



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class main:

	def __init__(self, q, limit=1, count=10):
		""" bing.com search engine
			
			q         : Query for search
			limit	  : Number of pages
			count	  : Number of links
		"""
		self.framework = main.framework
		self.q = self.framework.urlib(q).quote
		self.limit = limit
		self.count = count
		self._pages = ''
		self.bing = 'bing.com'

	def run_crawl(self):
		set_page = lambda x: x*11
		urls = [f"https://www.{self.bing}/search?q={self.q}&count={self.count}&\
					first={set_page(i)}&form=QBLH&pq={self.q.lower()}" for i in range(1, self.limit+1)]
		max_attempt = len(urls)
		for url in range(max_attempt):
			self.framework.verbose(f"[BING] Searching in {url} page...")
			try:
				req = self.framework.request(url=urls[url], allow_redirects=True)
			except:
				self.framework.error('ConnectionError', 'util/bing', 'run_crawl')
				max_attempt -= 1
				if max_attempt == 0:
					self.framework.error('Bing is missed!', 'util/bing', 'run_crawl')
					break
			else:
				page = req.text
				if 'title="Next page"' not in page:
					self._pages += page
					break
				self._pages += page

	@property
	def pages(self):
		return self._pages

	@property
	def dns(self):
		return self.framework.page_parse(self._pages).get_dns(self.q)

	@property
	def emails(self):
		return self.framework.page_parse(self._pages).get_emails(self.q)

	@property
	def sites(self):
		return self.framework.page_parse(self._pages).sites

	@property
	def links(self):
		parser = self.framework.page_parse(self._pages)
		parser.pclean
		links = parser.findall(r'<a target="_blank" href="([^"]+)" h="ID=SERP,[\d\.]+">')
		if len(links) <= 1:
			links += parser.findall(r'<a href="([^"]+)" h="ID=SERP,[\d\.]+">')
		links = [x for x in links if "http://www.microsofttranslator.com" not in x]
		return links

	@property
	def links_with_title(self):
		parser = self.framework.page_parse(self._pages)
		parser.pclean
		links = parser.findall(r'<a target="_blank" href="([^"]+)" h="ID=SERP,[\d\.]+">([^<]+)</a>')
		if len(links) <= 1:
			links = parser.findall(r'<a href="([^"]+)" h="ID=SERP,[\d\.]+">([^<]+)</a>')
		links = [x for x in links if x[0].startswith('http') and\
			"http://www.microsofttranslator.com" not in x[0] and\
			"Translate this page" not in x[1]]
		return links

	@property
	def docs(self):
		return self.framework.page_parse(self._pages).get_docs(self.q, self.links)



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import concurrent.futures

meta = {
	'name': 'Iris Meta Search Engine(experiment version)',
	'author': 'Saeed',
	'version': '0.2',
	'description': 'Iris is a built-in meta search engine.',
	'comments': ('It should be note that this is a beta version and has lots of bugs!',),
	'contributors': 'Aman, Dimitris, Divya, Vikas, Kunal',
	'sources': ('google', 'yahoo', 'bing', 'etools', 'metacrawler', 'searx', 'dogpile'),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
		('limit', 1, False, 'Search limit(number of pages, default=1)', '-l', 'store', int),
		('count', 50, False, 'Number of results per page (default=50)', '-c', 'store', int),
	),
	'examples': ('iris -q <QUERY> -l 15 --output --api',)
}
RESULTS = []
URLS = []
COUNT_CONSENSUS = None
LIMIT_CONSENSUS = None
COUNT = 0

def thread(self, function, engines, query):
	threadpool = concurrent.futures.ThreadPoolExecutor(max_workers=len(engines))
	futures = (threadpool.submit(function, self, engine, query) for engine in engines)
	counter = 1
	for _ in concurrent.futures.as_completed(futures):
		pass

def remove_dups(res):
	global URLS
	new = []
	for i in res:
		a = i['a'].lower()
		if a not in URLS:
			URLS.append(a)
			new.append(i)
	return res

def search(self, name, q):
	global RESULTS, COUNT
	engine = getattr(self, name)
	limit = LIMIT_CONSENSUS[name] or 1
	count = COUNT_CONSENSUS[name] * 2
	if count:
		if name == 'google':
			attr = engine(q, limit, count, 'legacy')
		else:
			attr = engine(q, limit)
		attr.run_crawl()
		results = remove_dups(attr.results)
		COUNT += len(results)
		if results:
			RESULTS.append(results)

def module_api(self):
	global COUNT_CONSENSUS, LIMIT_CONSENSUS, RESULTS
	query = self.options['query']
	limit = self.options['limit']
	count = self.options['count']
	engines = ['google', 'duckduckgo', 'dogpile']
	COUNT_CONSENSUS = self.meta_search_util.compute_count_consensus(engines, count)
	LIMIT_CONSENSUS = self.meta_search_util.compute_count_consensus(engines, limit)
	thread(self, search, engines, query)
	engines_len = len(RESULTS)
	simple_merge = []
	for i in range(len(min(RESULTS, key=len))):
		for e in range(engines_len):
			simple_merge.append(RESULTS[e%engines_len].pop(i))
	for i in RESULTS:
		for j in i:
			simple_merge.append(j)
	output = {'results': simple_merge}
	self.save_gather(output, 'iris/iris', query, output=self.options['output'])
	return output

def module_run(self):
	self.search_engine_results(module_api(self))
